@@ -1,6 +1,6 @@
 #!/usr/bin/env python3 -i
 #
-# Copyright 2013-2025 The Khronos Group Inc.
+# Copyright 2013-2026 The Khronos Group Inc.
 #
 # SPDX-License-Identifier: Apache-2.0
 """Base class for source/header/doc generators, as well as some utility functions."""
@@ -50,65 +50,50 @@ def enquote(s):
     return None
 
 
-def regSortCategoryKey(feature):
-    """Sort key for regSortFeatures.
-    Sorts by category of the feature name string:
+def regSortFeatures(orderedFeatureNames, features):
+    """Default sort procedure for generated features.
 
-    - Core API features (those defined with a `<feature>` tag)
-        - (sort VKSC after VK - this is Vulkan-specific)
-    - ARB/KHR/OES (Khronos extensions)
-    - other       (EXT/vendor extensions)"""
-
-    if feature.elem.tag == 'feature':
-        if feature.name.startswith('VKSC'):
-            return 0.5
-        else:
-            return 0
-
-    if feature.category.upper() in ('ARB', 'KHR', 'OES'):
-        return 1
-
-    return 2
-
-
-def regSortOrderKey(feature):
-    """Sort key for regSortFeatures - key is the sortorder attribute."""
-
-    return feature.sortorder
-
-
-def regSortNameKey(feature):
-    """Sort key for regSortFeatures - key is the extension name."""
-
-    return feature.name
-
-
-def regSortFeatureVersionKey(feature):
-    """Sort key for regSortFeatures - key is the feature version.
-    `<extension>` elements all have version number 0."""
-
-    return float(feature.versionNumber)
-
-
-def regSortExtensionNumberKey(feature):
-    """Sort key for regSortFeatures - key is the extension number.
-    `<feature>` elements all have extension number 0."""
-
-    return int(feature.number)
-
-
-def regSortFeatures(featureList):
-    """Default sort procedure for features.
+    - orderedFeatureNames - list of feature / extension names to sort
+    - features - dictionary of FeatureInfo keyed by names
 
     - Sorts by explicit sort order (default 0) relative to other features
     - then by feature category ('feature' or 'extension'),
     - then by version number (for features)
     - then by extension number (for extensions)"""
-    featureList.sort(key=regSortExtensionNumberKey)
-    featureList.sort(key=regSortFeatureVersionKey)
-    featureList.sort(key=regSortCategoryKey)
-    featureList.sort(key=regSortOrderKey)
 
+    # Sort by extension numbers; <feature>s all have extension number 0
+    orderedFeatureNames.sort(key=lambda name: int(features[name].number))
+
+    # Sort by feature version; <extension>s all have version number 0
+    orderedFeatureNames.sort(key=lambda name: float(features[name].versionNumber))
+
+    # Sort by feature name category
+    def categoryKey(feature):
+        """Helper function, too long to put in a sort key lambda
+
+        - feature - FeatureInfo containing the XML element to sort
+
+        Sorts by:
+
+        - Core API features (those defined with a `<feature>` tag)
+        - Sort VKSC core features after VK - Vulkan-specific
+        - ARB/KHR/OES (Khronos extensions)
+        - other       (EXT/vendor extensions)"""
+
+        if feature.elem.tag == 'feature':
+            if feature.name.startswith('VKSC'):
+                return 0.5
+            else:
+                return 0
+        elif feature.category.upper() in ('ARB', 'KHR', 'OES'):
+            return 1
+        else:
+            return 2
+
+    orderedFeatureNames.sort(key=lambda name: categoryKey(features[name]))
+
+    # Sort by sortorder attribute
+    orderedFeatureNames.sort(key=lambda name: features[name].sortorder)
 
 class MissingGeneratorOptionsError(RuntimeError):
     """Error raised when a Generator tries to do something that requires GeneratorOptions but it is None."""
@@ -1103,11 +1088,19 @@ class OutputGenerator:
             raise MissingGeneratorOptionsError()
         return self.genOpts.apientry + name + tail
 
-    def makeTypedefName(self, name, tail):
-        """Make the function-pointer typedef name for a command."""
+    def makeTypedefName(self, name, tail, isfuncpointer):
+        """Make the function-pointer typedef name for a command or
+           funcpointer name.
+           If this is a function pointer <type> tag, do not prepend a PFN_
+           to the name, as that is actually the type name and already
+           included.
+        """
         if self.genOpts is None:
             raise MissingGeneratorOptionsError()
-        return f"({self.genOpts.apientryp}PFN_{name}{tail})"
+
+        prefix = '' if isfuncpointer else 'PFN_'
+
+        return f'({self.genOpts.apientryp}{prefix}{name}{tail})'
 
     def makeCParamDecl(self, param, aligncol):
         """Return a string which is an indented, formatted
@@ -1335,11 +1328,15 @@ class OutputGenerator:
 
     def makeCDecls(self, cmd):
         """Return C prototype and function pointer typedef for a
-        `<command>` Element, as a two-element list of strings.
+        `<command>` or `type category="funcpointer"` Element, as a
+        two-element list of strings [prototype, typedef].
 
-        - cmd - Element containing a `<command>` tag"""
+        - cmd - Element containing a command or funcpointer tag"""
         if self.genOpts is None:
             raise MissingGeneratorOptionsError()
+
+        isfuncpointer = (cmd.tag == 'type')
+
         proto = cmd.find('proto')
         params = cmd.findall('param')
         # Begin accumulating prototype and typedef strings
@@ -1363,7 +1360,7 @@ class OutputGenerator:
             tail = noneStr(elem.tail)
             if elem.tag == 'name':
                 pdecl += self.makeProtoName(text, tail)
-                tdecl += self.makeTypedefName(text, tail)
+                tdecl += self.makeTypedefName(text, tail, isfuncpointer)
             else:
                 pdecl += text + tail
                 tdecl += text + tail
@@ -1378,8 +1375,8 @@ class OutputGenerator:
         # a <param> node without the tags. No tree walking required
         # since all tags are ignored.
         # Uses: self.indentFuncProto
-        # self.indentFuncPointer
-        # self.alignFuncParam
+        #       self.indentFuncPointer
+        #       self.alignFuncParam
         n = len(params)
         # Indented parameters
         if n > 0:
@@ -1419,7 +1416,12 @@ class OutputGenerator:
             paramdecl += 'void'
         paramdecl += ");"
 
-        return [pdecl + indentdecl, tdecl + paramdecl]
+        # For funcpointer types, only the typedef is returned,
+        # and it is formatted more attractively.
+        if isfuncpointer:
+            return [None, tdecl + indentdecl]
+        else:
+            return [pdecl + indentdecl, tdecl + paramdecl]
 
     def newline(self):
         """Print a newline to the output file (utility function)"""
